import 'package:dart_jwt/dart_jwt.dart';

class CustomPayloadClaimSet extends JwtClaimSet {
  final String subject;
  final String issuer;
  final List<String> audience;
  final DateTime issuedAt;
  final DateTime expiry;

  final String payload;

  CustomPayloadClaimSet(this.subject, this.issuer, this.audience, this.payload,
      {DateTime expiry, DateTime issuedAt})
      : issuedAt = issuedAt ?? new DateTime.now(),
        expiry = expiry ?? new DateTime.now().add(new Duration(minutes: 1));

  CustomPayloadClaimSet.fromJson(Map json)
      : subject = json["sub"],
        issuer = json["iss"],
        audience = json["aud"],
        issuedAt = new DateTime.fromMillisecondsSinceEpoch(json["iat"] * 1000),
        expiry = new DateTime.fromMillisecondsSinceEpoch(json["exp"] * 1000),
        payload = json['payload'];

  @override
  Set<ConstraintViolation> validate(
      CustomPayloadClaimSetValidationContext validationContext) {
    final violations = new Set<ConstraintViolation>();

    if (subject is! String || subject.isEmpty) {
      violations.add(new ConstraintViolation("Bad subject"));
    }
    if (issuer != validationContext.expectedIssuer) {
      violations.add(new ConstraintViolation("Invalid issuer: \"$issuer\""));
    }
    for (String aud in audience) {
      if (!validationContext.expectedAudience.contains(aud)) {
        violations
            .add(new ConstraintViolation("Invalid audience: \"$audience\""));
      }
    }

    final now = new DateTime.now();

    if (now.isBefore(issuedAt.subtract(validationContext.maxClockSkew))) {
      var diff = issuedAt.difference(now);
      violations
          .add(new ConstraintViolation("Issued in the future: $diff h:m:s"));
    }
    if (now.isAfter(expiry.add(validationContext.maxClockSkew))) {
      var diff = now.difference(expiry);
      violations.add(new ConstraintViolation("Expired: by $diff h:m:s"));
    }

    return violations;
  }

  Map toJson() {
    return {
      'sub': subject,
      'iss': issuer,
      'aud': audience,
      'iat': (issuedAt.millisecondsSinceEpoch ?? 0) ~/ 1000,
      'exp': (expiry.millisecondsSinceEpoch ?? 0) ~/ 1000,
      'payload': payload
    };
  }
}

class CustomPayloadClaimSetValidationContext
    extends JwtClaimSetValidationContext {
  final String expectedIssuer;
  final List<String> expectedAudience;
  final Duration maxClockSkew;

  CustomPayloadClaimSetValidationContext(
      this.expectedIssuer, this.expectedAudience,
      {Duration maxClockSkew: const Duration(seconds: 30)})
      : maxClockSkew = maxClockSkew,
        super(expiryTolerance: maxClockSkew);
}

class CustomPayloadValidationContext extends JwtValidationContext {
  CustomPayloadValidationContext(
      String sharedSecret, CustomPayloadClaimSetValidationContext csvc)
      : super(new JwaSymmetricKeySignatureContext(sharedSecret), csvc) {}
}

CustomPayloadClaimSet customPayloadClaimSetParser(Map json) {
  try {
    return new CustomPayloadClaimSet.fromJson(json);
  } catch (e) {
    throw new FormatException(
        "Bad CustomPayload Connect JWT: $e", json.toString());
  }
}
